#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QMap>
#include <QButtonGroup>
#include <QPushButton>
#include <QTcpSocket>
#include "MyTcpClient.h"
#include "data_config.h"
#include <QProgressBar>

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE



class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

private:
    Ui::MainWindow *ui;

    QAction * action_menu_monitors;
    QAction * action_menu_keyphasor;
    QAction * action_menu_relays;
    QAction * action_menu_reset;

    //声明菜单
    QMenu * button_menu;

    QMap<int,SlotConfig> map_slot_config;
    QList<QLabel*> list_label;

    QButtonGroup * btnGroup_slot = nullptr;
    MyTcpClient* m_tcpClient;

    int slot_no = 0;
    QTcpSocket *socket;

    QProgressBar *progressBar;
    CardType card_type;

    void createMenu();
    void createMenu(const QString& rootTitle, QPushButton* button = nullptr);
    void createMenuSet(const QString& rootTitle, QPushButton* button = nullptr);
    void clearMenuProperties(QMenu* menu);

    void readJsonFile(const QString &filePath);
    void sendUpgradePackage(int slot);
    void getVersion(int slot);
    void initStyle();
    void connectServer();
    void onerrorOccurred(const QString &errorMsg);
private slots:

    void onDisConnected();
    void onConnected();
    void readData(const QByteArray&);
    void onMenuAction_relay();
    void OnButtonGroup(QAbstractButton *);
    void onConnect();
    void onDisconnect();

    void onMenuActionTriggered();
    void on_pushButton_slot_clicked();
    void on_pushButton_chan_clicked();
    void on_pushButton_alarm_clicked();
    void on_pushButton_point_name_clicked();
    void on_pushButton_save_clicked();
    void on_pushButton_open_clicked();
    void on_pushButton_download_clicked();
};
#endif // MAINWINDOW_H
